==> evlan-0.3.3.tbz2: evlan-0.3.3/evlan-client-0.3.3/CommandStream.java <==
/* Evlan Client - Command shell for the Evlan programming language.
 * Copyright (C) 2004-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.io.*;
import java.net.Socket;
import java.util.ArrayList;

public class CommandStream
{
   // public interface ===============================================
   public CommandStream(Socket socket) throws IOException
   {
      mConnection = socket;
      mInput = mConnection.getInputStream();
      mOutput = mConnection.getOutputStream();

      mBuffer = new byte[8192];
      mBufferStart = 0;
      mBufferEnd = 0;
   }

   public String read() throws IOException
   {
      String line = readLine();
      if(line == null)
         return null;

      if(line.length() > 0 && line.charAt(0) == '>')
      {
         StringBuffer buffer = new StringBuffer();

         buffer.append(line.substring(1, line.length()));
         line = readLine();

         while(line.length() > 0 && line.charAt(0) == '>')
         {
            buffer.append("\n");
            buffer.append(line.substring(1, line.length()));
            line = readLine();
            if(line == null)
               return null;
         }

         line = buffer.toString();
      }

      return line;
   }

   public byte[] readData(int size) throws IOException
   {
      ArrayList<byte[]> data = readUntilSize(size);
      if(data == null)
         return null;
      return join(data);
   }

   public void write(String command) throws IOException
   {
      int pos = command.indexOf('\n') + 1;

      if(pos == 0 && (command.length() == 0 || command.charAt(0) != '>'))
         mOutput.write((command + "\n").getBytes("UTF-8"));
      else
      {
         StringBuffer buffer = new StringBuffer();
         int start = 0;

         while(pos != 0)
         {
            buffer.append(">");
            buffer.append(command.substring(start, pos));

            start = pos;
            pos = command.indexOf('\n', start) + 1;
         }

         buffer.append(">");
         buffer.append(command.substring(start));
         buffer.append("\n\n");

         mOutput.write(buffer.toString().getBytes("UTF-8"));
      }
      mOutput.flush();
   }

   public void writeData(byte[] data, int size) throws IOException
   {
      mOutput.write(data, 0, size);
   }

   public void close() throws IOException
   {
      mConnection.close();
   }

   // internal =======================================================
   private Socket mConnection;
   private OutputStream mOutput;
   private InputStream mInput;

   private byte[] mBuffer;
   private int mBufferStart;
   private int mBufferEnd;


   private byte[] getBufferInterval(int start, int end) throws IOException
   {
      byte[] newData = new byte[end - start];
      for(int j = 0; j < newData.length; j++)
         newData[j] = mBuffer[start + j];
      return newData;
   }

   private void refill() throws IOException
   {
      mBufferStart = 0;
      mBufferEnd = mInput.read(mBuffer);
   }

   private String readLine() throws IOException
   {
      ArrayList<byte[]> data = readUntilNewline();
      if(data == null)
         return null;

      return new String(join(data), "UTF-8");
   }

   private byte[] join(ArrayList<byte[]> data)
   {
      int totalLength = 0;
      for(int i = 0; i < data.size(); i++)
         totalLength += data.get(i).length;

      byte[] allBytes = new byte[totalLength];
      totalLength = 0;

      for(int i = 0; i < data.size(); i++)
      {
         byte[] part = data.get(i);
         for(int j = 0; j < part.length; j++)
            allBytes[totalLength+j] = part[j];
         totalLength += part.length;
      }

      return allBytes;
   }

   private ArrayList<byte[]> readUntilNewline() throws IOException
   {
      ArrayList<byte[]> data = new ArrayList<byte[]>();

      while(true)
      {
         if(mBufferEnd == -1)
            return null;

         for(int i = mBufferStart; i < mBufferEnd; i++)
         {
            if(mBuffer[i] == '\n')
            {
               data.add(getBufferInterval(mBufferStart, i));
               mBufferStart = i + 1;
               return data;
            }
         }

         data.add(getBufferInterval(mBufferStart, mBufferEnd));
         refill();
      }
   }

   private ArrayList<byte[]> readUntilSize(int size) throws IOException
   {
      ArrayList<byte[]> data = new ArrayList<byte[]>();

      while(true)
      {
         if(mBufferEnd == -1)
            return null;

         if(mBufferEnd - mBufferStart >= size)
         {
            data.add(getBufferInterval(mBufferStart, mBufferStart + size));
            mBufferStart += size;
            return data;
         }

         data.add(getBufferInterval(mBufferStart, mBufferEnd));
         size -= mBufferEnd - mBufferStart;
         refill();
      }
   }
}

==> evlan-0.3.4.tbz2: evlan-0.3.4/evlan-client-0.3.4/CommandStream.java <==
/* Evlan Client - Command shell for the Evlan programming language.
 * Copyright (C) 2004-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.io.*;
import java.net.Socket;
import java.util.ArrayList;

public class CommandStream
{
   // public interface ===============================================
   public CommandStream(Socket socket) throws IOException
   {
      mConnection = socket;
      mInput = mConnection.getInputStream();
      mOutput = mConnection.getOutputStream();

      mBuffer = new byte[8192];
      mBufferStart = 0;
      mBufferEnd = 0;
   }

   public String read() throws IOException
   {
      String line = readLine();
      if(line == null)
         return null;

      if(line.length() > 0 && line.charAt(0) == '>')
      {
         StringBuffer buffer = new StringBuffer();

         buffer.append(line.substring(1, line.length()));
         line = readLine();

         while(line.length() > 0 && line.charAt(0) == '>')
         {
            buffer.append("\n");
            buffer.append(line.substring(1, line.length()));
            line = readLine();
            if(line == null)
               return null;
         }

         line = buffer.toString();
      }

      return line;
   }

   public byte[] readData(int size) throws IOException
   {
      ArrayList data = readUntilSize(size);
      if(data == null)
         return null;
      return join(data);
   }

   public void write(String command) throws IOException
   {
      int pos = command.indexOf('\n') + 1;

      if(pos == 0 && (command.length() == 0 || command.charAt(0) != '>'))
         mOutput.write((command + "\n").getBytes("UTF-8"));
      else
      {
         StringBuffer buffer = new StringBuffer();
         int start = 0;

         while(pos != 0)
         {
            buffer.append(">");
            buffer.append(command.substring(start, pos));

            start = pos;
            pos = command.indexOf('\n', start) + 1;
         }

         buffer.append(">");
         buffer.append(command.substring(start));
         buffer.append("\n\n");

         mOutput.write(buffer.toString().getBytes("UTF-8"));
      }
      mOutput.flush();
   }

   public void writeData(byte[] data, int size) throws IOException
   {
      mOutput.write(data, 0, size);
   }

   public void close() throws IOException
   {
      mConnection.close();
   }

   // internal =======================================================
   private Socket mConnection;
   private OutputStream mOutput;
   private InputStream mInput;

   private byte[] mBuffer;
   private int mBufferStart;
   private int mBufferEnd;


   private byte[] getBufferInterval(int start, int end) throws IOException
   {
      byte[] newData = new byte[end - start];
      for(int j = 0; j < newData.length; j++)
         newData[j] = mBuffer[start + j];
      return newData;
   }

   private void refill() throws IOException
   {
      mBufferStart = 0;
      mBufferEnd = mInput.read(mBuffer);
   }

   private String readLine() throws IOException
   {
      ArrayList data = readUntilNewline();
      if(data == null)
         return null;

      return new String(join(data), "UTF-8");
   }

   private byte[] join(ArrayList data)
   {
      int totalLength = 0;
      for(int i = 0; i < data.size(); i++)
         totalLength += ((byte[])data.get(i)).length;

      byte[] allBytes = new byte[totalLength];
      totalLength = 0;

      for(int i = 0; i < data.size(); i++)
      {
         byte[] part = (byte[])data.get(i);
         for(int j = 0; j < part.length; j++)
            allBytes[totalLength+j] = part[j];
         totalLength += part.length;
      }

      return allBytes;
   }

   private ArrayList readUntilNewline() throws IOException
   {
      ArrayList data = new ArrayList();

      while(true)
      {
         if(mBufferEnd == -1)
            return null;

         for(int i = mBufferStart; i < mBufferEnd; i++)
         {
            if(mBuffer[i] == '\n')
            {
               data.add(getBufferInterval(mBufferStart, i));
               mBufferStart = i + 1;
               return data;
            }
         }

         data.add(getBufferInterval(mBufferStart, mBufferEnd));
         refill();
      }
   }

   private ArrayList readUntilSize(int size) throws IOException
   {
      ArrayList data = new ArrayList();

      while(true)
      {
         if(mBufferEnd == -1)
            return null;

         if(mBufferEnd - mBufferStart >= size)
         {
            data.add(getBufferInterval(mBufferStart, mBufferStart + size));
            mBufferStart += size;
            return data;
         }

         data.add(getBufferInterval(mBufferStart, mBufferEnd));
         size -= mBufferEnd - mBufferStart;
         refill();
      }
   }
}
